<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Traits\MediaUploadingTrait;
use App\Http\Requests\MassDestroyLessonRequest;
use App\Http\Requests\StoreLessonRequest;
use App\Http\Requests\UpdateLessonRequest;
use App\Imports\LessonImport;
use App\Models\Course;
use App\Models\Lesson;
use App\Models\Trainer;
use App\Models\User;
use Carbon\Carbon;
use Gate;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Models\CourseRole;
use App\Models\TrainerApproval;
use Spatie\MediaLibrary\MediaCollections\Models\Media;
use Symfony\Component\HttpFoundation\Response;
use Excel;
use Illuminate\Support\Facades\DB;

class LessonController extends Controller
{
    use MediaUploadingTrait;
    
    public $courseRole;
    protected $user;


    public function __construct()
    {
        $this->middleware(function ($request, $next) {
            $this->user= Auth::user();
            $this->courseRole = CourseRole::where('user_id', $this->user->id)->get();
            return $next($request);
        });
    } 
    

    public function index()
    {
        abort_if(Gate::denies('lesson_access',$this->courseRole), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $lessons = Lesson::with(['course', 'resource_person'])->get();

        return view('admin.lessons.index', compact('lessons'));
    }

    public function create()
    {
        abort_if(Gate::denies('lesson_create',$this->courseRole), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $courses = Course::pluck('course_code', 'id')->prepend(trans('global.pleaseSelect'), '');

        $resource_people = Trainer::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        return view('admin.lessons.create', compact('courses', 'resource_people'));
    }

    public function store(StoreLessonRequest $request)
    {
        DB::beginTransaction();
        try {
            $lesson = Lesson::create($request->all());

            if ($media = $request->input('ck-media', false)) {
                Media::whereIn('id', $media)->update(['model_id' => $lesson->id]);
            }
            if($request->resource_person_id)
            {
                $user = User::find($request->resource_person_id);

                $exist_role=CourseRole::where("course_id",$request->course_id)->where('user_id',$user->id)->first();

                if(!$exist_role)
                {
                    $course_role = new CourseRole();
                    $course_role->user_id = $user->id;
                    $course_role->course_id = $request->course_id;
                    $course_role->role_id = 7;
                    $course_role->save();

                    $ta = new TrainerApproval();
                    $ta->user_id = $user->id;
                    $ta->course_id = $request->course_id;
                    $ta->approved = 0;
                    $ta->save();
                }
            }
            
            

            
        } catch (\Exception $e) {
            DB::rollback();

            throw $e;
        }
        DB::commit();
        return response()->json('success');
        // return redirect()->route('lessons.index');
    }
    // 
    public function lessonFileStore(Request $request)
    {
        $courseId = $request->course_id;
        $file = $request->file('file');
        $excel =  Excel::import(new LessonImport($courseId), $file);       
        return back();

    }

    public function edit(Lesson $lesson)
    {
        abort_if(Gate::denies('lesson_edit',$this->courseRole), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $courses = Course::pluck('course_code', 'id')->prepend(trans('global.pleaseSelect'), '');

        $resource_people = Trainer::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        $lesson->load('course', 'resource_person');

        return view('admin.lessons.edit', compact('courses', 'lesson', 'resource_people'));
    }

    public function update(UpdateLessonRequest $request, Lesson $lesson)
    {
        $lesson->update($request->all());

        return response()->json('success');
    }

    public function show(Lesson $lesson)
    {
        abort_if(Gate::denies('lesson_show',$this->courseRole), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $lesson->load('course', 'resource_person');

        return view('admin.lessons.show', compact('lesson'));
    }

    public function destroy(Lesson $lesson)
    {
        abort_if(Gate::denies('lesson_delete',$this->courseRole), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $lesson->delete();

        return back();
    }

    public function massDestroy(MassDestroyLessonRequest $request)
    {
        Lesson::whereIn('id', request('ids'))->delete();

        return response(null, Response::HTTP_NO_CONTENT);
    }

    public function storeCKEditorImages(Request $request)
    {
        abort_if(Gate::denies('lesson_create') && Gate::denies('lesson_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $model         = new Lesson();
        $model->id     = $request->input('crud_id', 0);
        $model->exists = true;
        $media         = $model->addMediaFromRequest('upload')->toMediaCollection('ck-media');

        return response()->json(['id' => $media->id, 'url' => $media->getUrl()], Response::HTTP_CREATED);
    }
    
}
