<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Requests\MassDestroyNominationRequest;
use App\Http\Requests\StoreNominationRequest;
use App\Http\Requests\UpdateNominationRequest;
use App\Jobs\NominationMailJob;
use App\Models\Batch;
use App\Models\Course;
use App\Models\Nomination;
use App\Models\Setting;
use Gate;
use Illuminate\Support\Facades\Auth;
use App\Models\CourseRole;
use App\Models\NominatedOrganisation;
use App\Models\Organization;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Symfony\Component\HttpFoundation\Response;

class NominationController extends Controller
{
    public $courseRole;

    public function __construct()
    {
        $this->middleware(function ($request, $next) {
            $user = Auth::user();
            $this->courseRole = CourseRole::where('user_id', $user->id)->get();
            return $next($request);
        });
    }

    public function index()
    {
        abort_if(Gate::denies('nomination_access', $this->courseRole), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $nominations = Nomination::with(['course'])->latest()->get();

        return view('admin.nominations.index', compact('nominations'));
    }

    public function create()
    {
        abort_if(Gate::denies('nomination_create', $this->courseRole), Response::HTTP_FORBIDDEN, '403 Forbidden');
        $organizations = Organization::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');
        $settings = Setting::first();
        $courses = Course::where('is_approved', 1)->latest()->get();
        return view('admin.nominations.create', compact('settings', 'organizations', 'courses'));
    }

    public function store(Request $request)
    {
        // dd($request->all());
        $request->validate([
            'subject' => 'required',
        ]);
        DB::beginTransaction();
        try {

            $nomination = new Nomination();
            $nomination->subject = $request->subject;
            $nomination->course_id = $request->course_id;
            $nomination->course_fee = $request->course_fee;
            if ($request->has('file')) {
                $fileName = time() . '.' . $request->file->extension();
                $request->file->move(public_path('upload/files'), $fileName);
                $nomination->file = $fileName;
            }
            $nomination->save();

            //save org and number trainee to the database and sent mail
            if ($request->organization_id) {
                foreach ($request->organization_id as $key => $value) {
                    $nominated_org = new NominatedOrganisation();
                    $nominated_org->organization_id = $value;
                    $nominated_org->nomination_id  = $nomination->id;
                    $nominated_org->number_of_trainee = $request->number_of_trainee[$key];
                    $nominated_org->save();

                    //$findOrganization = Organization::where('id', $value)->first();
                    //$details['email'] = $findOrganization->email;

                    // ===== START SEND NOMINATION EMAIL =================
                    //dispatch(new NominationMailJob($details));
                    //Mail::to($findOrganization->email)->send(new NominationMail());

                }
            }

            Course::findOrFail($request->course_id)->update([
                'nomination_id' => $nomination->id
            ]);
        } catch (\Exception $e) {
            DB::rollback();
            throw $e;
        }

        DB::commit();
        $notification = array(
            'toaster-message' => 'Nomination created successfuly',
            'alert-type' => 'success'
        );
        return redirect()->route('nominations.index')->with($notification);
    }

    public function edit(Nomination $nomination)
    {
        abort_if(Gate::denies('nomination_edit', $this->courseRole), Response::HTTP_FORBIDDEN, '403 Forbidden');

        return view('admin.nominations.edit', compact('nomination'));
    }

    public function update(Request $request, Nomination $nomination)
    {

        if ($request->has('file')) {
            $fileName = time() . '.' . $request->file->extension();
            $request->file->move(public_path('upload/files'), $fileName);
            $nomination->file = $fileName;
        }
        if ($request->has('course_fee')) {
            $nomination->course_fee = $request->course_fee;
        }

        $nomination->save();

        $notification = array(
            'toaster-message' => 'Nomination Updated successfuly',
            'alert-type' => 'success'
        );
        return redirect()->route('nominations.index')->with($notification);
    }

    public function show(Nomination $nomination)
    {
        abort_if(Gate::denies('nomination_show', $this->courseRole), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $nomination->load(['course', 'nominationNominatedOrganisations' => function ($q) {
            $q->with('organization');
        }]);
        return view('admin.nominations.show', compact('nomination'));
    }

    public function destroy(Nomination $nomination)
    {
        abort_if(Gate::denies('nomination_delete', $this->courseRole), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $nomination->delete();

        return back();
    }

    public function massDestroy(MassDestroyNominationRequest $request)
    {
        Nomination::whereIn('id', request('ids'))->delete();

        return response(null, Response::HTTP_NO_CONTENT);
    }

    public function updateNumberTrainee($traineeId, Request $request)
    {
        $validated = $request->validate([
            'number_of_trainee' => 'required|integer',
        ]);

        $trainee = NominatedOrganisation::findOrFail($traineeId);

        $trainee->number_of_trainee = $validated['number_of_trainee'];
        $trainee->save();

        return response()->json(['message' => 'Number of trainees updated successfully!'], 200);
    }
}
