<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Traits\MediaUploadingTrait;
use App\Http\Requests\MassDestroyOrganizationRequest;
use App\Http\Requests\StoreOrganizationRequest;
use App\Http\Requests\UpdateOrganizationRequest;
use App\Jobs\TraineeApproveJob;
use App\Models\NominatedTrainee;
use App\Models\Nomination;
use App\Models\Organization;
use Gate;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Spatie\MediaLibrary\MediaCollections\Models\Media;
use Symfony\Component\HttpFoundation\Response;
use App\Mail\TraineeMail;
use App\Models\Course;
use App\Models\User;
use Illuminate\Support\Facades\Auth;
use App\Models\CourseRole;
use Illuminate\Support\Facades\Mail;

class OrganizationController extends Controller
{
    use MediaUploadingTrait;

    public $courseRole;

    public function __construct()
    {
        $this->middleware(function ($request, $next) {
            $user = Auth::user();
            $this->courseRole = CourseRole::where('user_id', $user->id)->get();
            return $next($request);
        });
    }


    public function index()
    {
        abort_if(Gate::denies('organization_access', $this->courseRole), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $organizations = Organization::with(['media'])->get();

        return view('admin.organizations.index', compact('organizations'));
    }

    public function create()
    {
        abort_if(Gate::denies('organization_create', $this->courseRole), Response::HTTP_FORBIDDEN, '403 Forbidden');

        return view('admin.organizations.create');
    }

    public function store(StoreOrganizationRequest $request)
    {
        $organization = Organization::create($request->all());

        if ($request->input('logo', false)) {
            $organization->addMedia(storage_path('tmp/uploads/' . basename($request->input('logo'))))->toMediaCollection('logo');
        }

        if ($media = $request->input('ck-media', false)) {
            Media::whereIn('id', $media)->update(['model_id' => $organization->id]);
        }

        return redirect()->route('organizations.index');
    }

    public function edit(Organization $organization)
    {
        abort_if(Gate::denies('organization_edit', $this->courseRole), Response::HTTP_FORBIDDEN, '403 Forbidden');

        return view('admin.organizations.edit', compact('organization'));
    }

    public function update(UpdateOrganizationRequest $request, Organization $organization)
    {
        $organization->update($request->all());

        if ($request->input('logo', false)) {
            if (!$organization->logo || $request->input('logo') !== $organization->logo->file_name) {
                if ($organization->logo) {
                    $organization->logo->delete();
                }
                $organization->addMedia(storage_path('tmp/uploads/' . basename($request->input('logo'))))->toMediaCollection('logo');
            }
        } elseif ($organization->logo) {
            $organization->logo->delete();
        }

        return redirect()->route('organizations.index');
    }

    public function show(Organization $organization)
    {
        abort_if(Gate::denies('organization_show', $this->courseRole), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $organization->load('organizationNominatedOrganisations', 'organizationNominatedTrainees');

        return view('admin.organizations.show', compact('organization'));
    }

    public function destroy(Organization $organization)
    {
        abort_if(Gate::denies('organization_delete', $this->courseRole), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $organization->delete();

        return back();
    }

    public function massDestroy(MassDestroyOrganizationRequest $request)
    {
        Organization::whereIn('id', request('ids'))->delete();

        return response(null, Response::HTTP_NO_CONTENT);
    }

    public function storeCKEditorImages(Request $request)
    {

        $model         = new Organization();
        $model->id     = $request->input('crud_id', 0);
        $model->exists = true;
        $media         = $model->addMediaFromRequest('upload')->toMediaCollection('ck-media');

        return response()->json(['id' => $media->id, 'url' => $media->getUrl()], Response::HTTP_CREATED);
    }

    public function traineeList()
    {
        $courses =  Course::whereNotNUll('nomination_id')->whereHas('courseRoles',function($q){
            $q->where('role_id','=',9)->where('user_id',auth()->user()->id);
        })->latest()->get();
        
        return view('admin.approve-list.index', compact('courses'));
    }



    public function courseWiseTrainee($courseid)
    {
        $course= Course::findOrFail($courseid);

        $nominatedTrainees = NominatedTrainee::where('nomination_id', $course->nomination_id)->with('user', 'organization')->get()->groupBy(function ($val) {
            return $val->organization->name;
        });
        return view('admin.approve-list.orgList', compact('nominatedTrainees'));
    }
    // TRAINEE APPROVED BY ADMIN
    public function approveTrainee(Request $request)
    {

        $trainee_id = $request->trainee_id;
        
        DB::beginTransaction();
        try {
            if ($request->type == 'single') {
                $this->updateTraineeAndSendMail($trainee_id);
            } else {
                foreach ($trainee_id as $key => $value) {
                    $this->updateTraineeAndSendMail($value);
                }
            }

            DB::commit();
            $notification = array(
                'toaster-message' => 'Trainee approved successfuly',
                'alert-type' => 'success'
            );
            // return redirect()->back()->with($notification);
            // return back();
        } catch (\Exception $e) {
            DB::rollBack();
            dd($e->getMessage());
        }
    }
    // TRAINEE DENY 
    public function denyTrainee(Request $request)
    {
        DB::beginTransaction();   
        try{
            $findNom = NominatedTrainee::find($request->nomination_id);
            $findNom->update([
                'approve'=>3
            ]); 
            DB::commit();
            $notification = array(
                'toaster-message' => 'Trainee Deny successfuly',
                'alert-type' => 'success'
            );
        }catch(\Exception $e){
            DB::rollBack();
            dd($e->getMessage());
        }
    }

    public function updateTraineeAndSendMail($nominationId)
    {
        $nominatedTrainees = NominatedTrainee::findOrFail($nominationId);
        $nominatedTrainees->update(['approve' => 1]);

        // dispatch(new TraineeApproveJob($nominatedTrainees));
    }
}
