<?php

namespace App\Http\Controllers;

use App\Models\Course;
use App\Models\ExamAttend;
use App\Models\ExamType;
use App\Models\QuizCategory;
use App\Models\Quiz;
use App\Models\QuizQuestion;
use Illuminate\Http\Request;
use Gate;
use App\Models\CourseRole;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Symfony\Component\HttpFoundation\Response;

class QuizController extends Controller
{
    public $courseRole;

    public function __construct()
    {

        $this->middleware(function ($request, $next) {
            $user = Auth::user();
            $this->courseRole = CourseRole::where('user_id', $user->id)->get();
            return $next($request);
        });
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        abort_if(Gate::denies('quiz_access', $this->courseRole), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $quizes = Quiz::with(['course', 'module'])->latest()->paginate(20);
        return view('quiz.index', compact('quizes'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $cats = QuizCategory::where('parent_id', 0)->get();
        $courses = Course::where('is_approved', 1)->latest()->get();
        $examTypes = ExamType::latest()->get();
        return view('quiz.create', compact('cats', 'courses', 'examTypes'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $Quiz = new Quiz();
        $Quiz->title = $request->title;
        $Quiz->course_id = $request->course_id;
        $Quiz->module_id = $request->module;
        $Quiz->topic = $request->topic;
        $Quiz->quiz_mark = $request->quiz_mark;
        $Quiz->exam_module_id = $request->exam_module_id;
        $Quiz->time_limit = $request->time_limit;
        $Quiz->description = $request->description;
        $Quiz->date_time = $request->date_time;
        $Quiz->exam_type = $request->exam_type;
        $Quiz->created_by = auth()->user()->id;
        $Quiz->save();

        foreach ($request->questions as $key => $question) {
            $qq = new QuizQuestion();
            $qq->quizes_id = $Quiz->id;
            $qq->question_banks_id = $question;
            $qq->save();
        }
        $notification = array(
            'toaster-message' => 'Quiz added successfuly',
            'alert-type' => 'success'
        );

        return redirect()->route('quiz.index.index')->with($notification);
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Quiz  $Quiz
     * @return \Illuminate\Http\Response
     */
    public function show(Quiz $Quiz)
    {
        //
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\Models\Quiz  $Quiz
     * @return \Illuminate\Http\Response
     */
    public function viewCourseQuiz(Course $course)
    {
        abort_if(Gate::denies('course_access', $this->courseRole), Response::HTTP_FORBIDDEN, '403 Forbidden');

        if (Gate::allows('trainer_evaluation', $this->courseRole)) {
            $courseinfo = $course->load(['quizes' => function ($query) {
                $query->with(['exam_attends' => function ($q) {
                    $q->where('user_id', auth()->user()->id);
                }]);
            }]);
        } else {
            $courseinfo = $course->load(['quizes' => function ($query) {
                $query->with(['exam_attends']);
            }]);
            //dd($courseinfo);
        }

        //dd($courseinfo);
        return view('quiz.quizList', compact('courseinfo'));
    }

    public function viewQuizExam($quizid)
    {
        $exam_attend = new ExamAttend();
        $exam_attend->quiz_id = $quizid;
        $exam_attend->user_id = auth()->user()->id;
        $exam_attend->status = 1;
        $exam_attend->save();

        $quiz = Quiz::find($quizid)->load('questions');
        return view('quiz.start-exam', compact('quiz'));
    }



    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\Models\Quiz  $Quiz
     * @return \Illuminate\Http\Response
     */
    public function edit(Request $request)
    {
        $quiz = Quiz::where('id', $request->index)->first();
        $cats = QuizCategory::where('parent_id', 0)->get();
        $courses = Course::where('is_approved', 1)->latest()->get();
        $examTypes = ExamType::latest()->get();

        return view('quiz.edit', compact('quiz', 'cats', 'courses', 'examTypes'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Quiz  $Quiz
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request)
    {
        $Quiz = Quiz::where('id', $request->index)->first();
        $Quiz->title = $request->title;
        $Quiz->course_id = $request->course_id;
        $Quiz->module_id = $request->module;
        $Quiz->topic = $request->topic;
        $Quiz->quiz_mark = $request->quiz_mark;
        $Quiz->exam_module_id = $request->exam_module_id;
        $Quiz->time_limit = $request->time_limit;
        $Quiz->description = $request->description;
        $Quiz->date_time = $request->date_time;
        $Quiz->exam_type = $request->exam_type;
        $Quiz->created_by = auth()->user()->id;
        $Quiz->update();

        foreach ($request->questions as $key => $question) {
            $qq = new QuizQuestion();
            $qq->quizes_id = $Quiz->id;
            $qq->question_banks_id = $question;
            $qq->save();
        }
        $notification = array(
            'toaster-message' => 'Quiz updated successfuly',
            'alert-type' => 'success'
        );

        return redirect()->route('quiz.index.index')->with($notification);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Quiz  $Quiz
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request)
    {
        $Quiz = Quiz::where('id', $request->index)->first();
        $Quiz->delete();
        $notification = array(
            'toaster-message' => 'Quiz deleted successfuly',
            'alert-type' => 'success'
        );

        return redirect()->route('quiz.index.index')->with($notification);
    }
}
