<?php

namespace App\Models;

use \DateTimeInterface;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Spatie\MediaLibrary\HasMedia;
use Spatie\MediaLibrary\InteractsWithMedia;
use Spatie\MediaLibrary\MediaCollections\Models\Media;

class Course extends Model implements HasMedia
{
    use InteractsWithMedia;
    use HasFactory;

    public const TYPE_COURSE_SELECT = [
        '1' => 'Basic',
        '2' => 'Refresher',
        '3' => 'Intermediate',
        '4' => 'Advanced',
        '5' => 'Special',
        '6' => 'Foundation',
    ];

    public $table = 'courses';

    protected $dates = [
        'course_start',
        'course_end',
        'created_at',
        'updated_at',
        'deleted_at',
    ];

    protected $fillable = [
        'course_code',
        'title',
        'type_course',
        'course_duration',
        'field_visit',
        'course_start',
        'course_end',
        'requiring_body',
        'course_principal_advisor',
        'nomination_id',
        'batch_id',
        'course_advisor',
        'course_director',
        'course_co_ordinator',
        'associcate_course_co_ordinator',
        'course_helper',
        'possible_trainers',
        'mds_approval',
        'mds_comment',
        'total_budget',
        'budget_tax',
        'total_budget_inc_vat',
        'rector_approval',
        'rector_comments',
        'is_approved',
        'comment',
        'created_by_id',
        'created_by',
        'created_at',
        'updated_at',
        'deleted_at',
    ];
    protected $casts = [
        'possible_trainers' => 'array',
        'course_principal_advisor' => 'array',
        'course_advisor' => 'array',
        'course_director' => 'array',
        'course_co_ordinator' => 'array',
        'associcate_course_co_ordinator' => 'array',
        'course_helper' => 'array'
    ];

    public function registerMediaConversions(Media $media = null): void
    {
        $this->addMediaConversion('thumb')->fit('crop', 50, 50);
        $this->addMediaConversion('preview')->fit('crop', 120, 120);
    }

    public function courseLessons()
    {
        return $this->hasMany(Lesson::class, 'course_id', 'id');
    }

    public function courseContents()
    {
        return $this->hasMany(CourseContent::class, 'course_id', 'id');
    }

    public function trainerApproval()
    {
        return $this->hasMany(TrainerApproval::class, 'course_id', 'id');
    }

    public function courseBudgets()
    {
        return $this->hasMany(CourseBudget::class, 'course_id', 'id');
    }

    public function courseRoles()
    {
        return $this->hasMany(CourseRole::class, 'course_id');
    }

    public function batch()
    {
        return $this->belongsTo(Batch::class, 'batch_id');
    }
    public function settings()
    {
        return $this->hasOne(CourseSetting::class, 'course_id');
    }

    public function courseAssociateOfficers()
    {
        return $this->hasMany(CourseAssociateOfficer::class, 'course_id', 'id');
    }

    public function getCourseStartAttribute($value)
    {
        return $value ? Carbon::parse($value)->format(config('panel.date_format')) : null;
    }

    public function setCourseStartAttribute($value)
    {
        $this->attributes['course_start'] = $value ? Carbon::createFromFormat(config('panel.date_format'), $value)->format('Y-m-d') : null;
    }

    public function getCourseEndAttribute($value)
    {
        return $value ? Carbon::parse($value)->format(config('panel.date_format')) : null;
    }

    public function setCourseEndAttribute($value)
    {
        $this->attributes['course_end'] = $value ? Carbon::createFromFormat(config('panel.date_format'), $value)->format('Y-m-d') : null;
    }

    public function setCreatedByIdAttribute($value)
    {
        $this->attributes['created_by_id'] = auth()->user()->id;
    }

    public function nomination()
    {
        return $this->hasOne(Nomination::class, 'course_id');
    }

    public function created_by()
    {
        return $this->belongsTo(User::class, 'created_by_id');
    }

    protected function serializeDate(DateTimeInterface $date)
    {
        return $date->format('Y-m-d H:i:s');
    }

    public function quizes()
    {
        return $this->hasMany(Quiz::class, 'course_id');
    }
    public function possibleTrainers()
    {
        return $this->belongsTo(User::class, 'possible_trainers');
    }

    public function payment()
    {
        return $this->hasOne(Coursefeepayment::class, 'course_id');
    }

    public function adjustment_sheet()
    {
        return $this->hasOne(AdjustmentSheet::class, 'course_id');
    }

    public function course_init()
    {
        return $this->belongsTo(CourseInitiation::class, 'course_initiation_id');
    }

    public function modules()
    {
        return $this->hasMany(Module::class, 'course_id');
    }

    public function courseExamModule()
    {
        return $this->hasMany(ExamModule::class, 'course_id');
    }

    public function courseAssignmentGroup()
    {
        return $this->hasMany(AssignmentGroup::class, 'course_id');
    }

    public function courseAssignmentFile()
    {
        return $this->hasMany(AssignmentFile::class, 'course_id');
    }

    public function coursePresentationGroup()
    {
        return $this->hasMany(PresentationGroup::class, 'course_id');
    }

    public function coursePresentation()
    {
        return $this->hasMany(PresentationFile::class, 'course_id');
    }

    public function releaseOrder()
    {
        return $this->hasOne(ReleaseOrder::class, 'course_id');
    }

    public function scopeAllList($query)
    {
        return $query->orderBy('id', 'desc');
    }


    public function scopeFilteredList($query)
    {
        $authid = strval(auth()->user()->id);

        return $query->leftJoin('nominations', 'nominations.id', '=', 'courses.nomination_id')
            ->leftJoin('nominated_trainees', 'nominated_trainees.nomination_id', '=', 'nominations.id')
            ->where('nominated_trainees.user_id', '=', auth()->user()->id)
            ->where('nominated_trainees.approve', '=', 1)
            ->leftJoin('trainer_approvals', 'trainer_approvals.course_id', 'courses.id')
            ->orWhere('trainer_approvals.user_id', auth()->user()->id)
            ->where('trainer_approvals.approved', '!=', 2)
            ->orWhereNull('trainer_approvals.approved')
            ->whereJsonContains('courses.possible_trainers', $authid)
            ->orWhereJsonContains('course_director', $authid)
            ->groupBy('courses.id')
            ->orderBy('courses.id', 'desc')
            ->select(
                'courses.id',
                'courses.course_code',
                'courses.title',
                'courses.type_course',
                'courses.course_duration',
                'courses.field_visit',
                'courses.course_start',
                'courses.course_end',
                'courses.is_approved',
                'courses.nomination_id',
                'courses.course_director',
                'courses.mds_approval',
                'courses.is_approved',
                'courses.rector_approval'
            );
    }
}
